import abc


class ClickAction(abc.ABC):
    @abc.abstractproperty
    def click_action_type(self):
        pass

    @abc.abstractproperty
    def click_action(self):
        pass


class NoAction(ClickAction):
    @property
    def click_action(self):
        return None

    @property
    def click_action_type(self):
        return 'no-action'


class LaunchApplication(ClickAction):

    def __init__(self, package_name=None):
        assert type(package_name) is strcli
        self.package_name = package_name

    @property
    def click_action(self):
        return self.package_name

    @property
    def click_action_type(self):
        return 'launch_application'


class OpenUrl(ClickAction):

    def __init__(self, url):
        assert url is not None
        assert type(url) == str
        assert len(url) > 0
        self.url = url

    @property
    def click_action(self):
        return self.url

    @property
    def click_action_type(self):
        return 'open_url'


class Telegram(ClickAction):

    def __init__(self, handle):
        assert handle is not None
        assert type(handle) == str
        assert len(handle) > 0

        self.handle = handle

    @property
    def click_action(self):
        return self.handle

    @property
    def click_action_type(self):
        return 'telegram'


class Instagram(ClickAction):
    def __init__(self, handle):
        assert handle is not None
        assert type(handle) == str
        assert len(handle) > 0

        self.handle = handle

    @property
    def click_action(self):
        return self.handle

    @property
    def click_action_type(self):
        return 'instagram'


class _JoinTelegramMixin(ClickAction):

    def __init__(self, link):
        assert link is not None
        assert type(link) == str
        assert len(link) > 0

        self.link = link

    @property
    def click_action_type(self):
        return 'join_telegram'

    @property
    def click_action(self):
        return self.link


class JoinTelegramGroup(_JoinTelegramMixin):
    pass


class JoinTelegramChannel(_JoinTelegramMixin):
    pass



import json
import logging

import re

from .push_click_action import NoAction, ClickAction

logger = logging.getLogger("nivad")


class NotificationObject:
    def __init__(self, title, text, **kwargs):
        self.title = title
        self.text = text
        self.full_title = str(kwargs.get('full_title', ''))
        self.full_text = str(kwargs.get('full_text', ''))
        self.status_bar_text = str(kwargs.get('status_bar_text', ''))
        self.vibration = kwargs.get('vibration', None)
        self.click_action = kwargs.get('click_action', NoAction())
        self.led_color = kwargs.get('led_color', None)

    def is_valid(self):
        """
        Checks fields values

        :return:
        """
        assert isinstance(self.click_action, ClickAction)

        if self.led_color:
            if type(self.led_color) == int:
                self.led_color = format(self.led_color, 'x')
            if not type(self.led_color) == str:
                self.led_color = None
                logger.warning('Invalid LED color type. should be string or integer')
            else:
                if len(self.led_color) == 7:
                    self.led_color = '0{}'.format(self.led_color)
                else:
                    self.led_color = '{}{}'.format('0' * (max(0, 6 - len(self.led_color))), self.led_color)

                if not re.match(r'[0-9A-Fa-f]{6}([0-9A-Fa-f]{2})?', self.led_color):
                    logger.warning('Invalid LED color')
                    self.led_color = None

        if self.vibration:
            if type(self.vibration) != bool:
                logger.warning('Invalid vibration value')
                self.vibration = None

        if not self.title or not self.text:
            logger.Error('Both Title and Text values are required')
            return False

        return True

    def to_dict(self):
        """
        Convert this notification message object to REST API compatible dictionary

        :return:
        """
        return_value = dict()
        for item in ['title', 'text', 'full_title', 'full_text', 'status_bar_text', 'vibration', 'led_color']:
            item_value = getattr(self, item, None)
            if item_value:
                return_value[item] = item_value

        return_value['click_action_type'] = self.click_action.click_action_type
        return_value['click_action'] = self.click_action.click_action
        if not return_value['click_action']:
            del return_value['click_action']

        return return_value


class NivadNotificationAPI:
    """
    Wrapper for notifications API Endpoint
    """
    URLS = {
        'send_notification': 'notification/'
    }

    def __init__(self, nivad_api):
        self.api = nivad_api

    def send(self, notification_object):
        """
        Send a notification to all devices

        :param notification_object:
        :return:
        """
        assert notification_object.is_valid(), 'Notification object is invalid'

        response = self.api.post(
            NivadNotificationAPI.URLS['send_notification'],
            data=json.dumps(notification_object.to_dict()),
            token_type='push_api'
        )

        return json.loads(response.data.decode('utf-8'))

    def get_info(self, notification_id):
        assert notification_id is not None and type(notification_id) == int

        response = json.loads(self.api.get(
            NivadNotificationAPI.URLS['send_notification'],
            data={'notification_id': str(notification_id)},
            token_type='push_api'
        ).data.decode('utf-8'))
        if not response['success']:
            raise ValueError('Invalid notification id')

        return NotificationInfo(response)


class NotificationInfo:

    def __init__(self, response_data):
        self._response_data = response_data

    def __getattr__(self, item):
        if item in self._response_data and item != 'message':
            return self._response_data[item]
        else:
            if 'message' in item:
                if len(item) < len('message_'):
                    raise AttributeError("Unknown attribute: '%s'" % item)
                item = item[len('message_'):]
                if item not in self._response_data['message']:
                    raise AttributeError("Unknown attribute: 'message_%s'" % item)
                else:
                    return self._response_data['message'][item]
            raise AttributeError("Unknown attribute: '%s'" % item)


class NotificationLEDColor:
    white = 'ffffff'
    red = 'ff0000'
    orange = 'fe9400'
    yellow = 'ffff00'
    pink = 'f90368'
    blue = '0000ff'
    purple = '8600ff'
    cyan = '00ffe6'
    green = '00ff00'
    lime = 'd2ff00'

"""
import nivad
from nivad.push_click_action import *
from nivad.push import *

api = nivad.NivadAPI('5d9dc126-8583-4d0e-b325-b76d107a8d32', push_api_secret='YmMolkX2PweqqkTeIOjDQMX4KuITTZeTr3JcV6bFz9jXoDrUrYSDHnM9RKTNvfUV')
notification = api.get_notification_api()
message = NotificationObject('title', 'text', full_title='full title', full_text='full text', vibration=False, status_bar_text='sb text', led_color=NotificationLEDColor.red)
notification.send(message)

"""


import json
import logging

import certifi
import jwt
import urllib3

from nivad.push import NivadNotificationAPI

DEBUG = True
logger = logging.getLogger("nivad")


def _get_connection_url():
    return '/api/v1/' if DEBUG else '/v1/'


def _get_connection_pool():
    if DEBUG:
        return urllib3.HTTPConnectionPool('127.0.0.1:8000')
    else:
        return urllib3.HTTPSConnectionPool(
            'api.nivad.io',
            cert_reqs='CERT_REQUIRED',
            ca_certs=certifi.where()
        )


class NivadAPI:
    base_url = _get_connection_url()
    connection_pool = _get_connection_pool()

    def __init__(self, application_id, **kwargs):
        self.application_id = application_id
        self.secrets = {
            'push': kwargs.get('push_secret', None),
            'push_api': kwargs.get('push_api_secret', None),
            'billing': kwargs.get('billing_secret', None),
            'master': kwargs.get('master_secret', None),
        }

    def get_notification_api(self):
        return NivadNotificationAPI(self)

    def post(self, url, data, token_type='master', extra_headers=None):
        return self.__request('post', url, data, token_type, extra_headers)

    def get(self, url, data, token_type='master', extra_headers=None):
        return self.__request('get', url, data, token_type, extra_headers)

    def put(self, url, data, token_type='master', extra_headers=None):
        return self.__request('put', url, data, token_type, extra_headers)

    def delete(self, url, data, token_type='master', extra_headers=None):
        return self.__request('delete', url, data, token_type, extra_headers)

    def __request(self, method, url, data, token_type='master', extra_headers=None):
        if not extra_headers:
            extra_headers = dict()

        assert type(extra_headers) == dict
        assert token_type in self.secrets

        method = method.upper()

        secret = (token_type, self.secrets[token_type])
        if not secret[1]:
            secret = ('master', self.secrets['master'])

        assert secret[1], 'Please provide appropriate secrets'

        authorization_token = jwt.encode({
            'typ': secret[0],
            'aid': self.application_id
        }, secret[1]).decode('utf-8')

        headers = {
            'Content-Type': 'application/json',
            'Authorization': 'bearer %s' % str(authorization_token)
        }

        if 'Authorization' in extra_headers:
            del extra_headers['Authorization']

        headers.update(extra_headers)

        request_url = '%s%s' % (_get_connection_url(), url)

        logger.info("Making %s request to %s -- Headers: %s" % (method, request_url, headers))
        if method == 'GET':
            response = NivadAPI.connection_pool.request(method, request_url, headers=headers, fields=data)
        else:
            if not type(data) == str:
                data = json.dumps(data)
            data = data.encode('utf-8')
            response = NivadAPI.connection_pool.request(method, request_url, headers=headers, body=data)

        return response

__all__ = [NivadAPI, NivadNotificationAPI]



from setuptools import setup

setup(
    version='0.0.1',
    name='nivad',
    description='Nivad server-server api package',
    url='https://github.com/NivadCloud/NivadPy',
    author='MJafar Mashhadi',
    author_email='mjafar@nivad.io',
    license='GPL',
    packages=['nivad'],
    install_requires=[
        'urllib3',
        'PyJWT',
        'certifi'
    ],
    zip_safe=False
)
